@@ -10,22 +10,15 @@ import jax.numpy as jnp
 import numpy as np
 import sympy
 
-from e3nn_jax import Irreps, IrrepsData, clebsch_gordan
+from e3nn_jax import Irreps, IrrepsArray, clebsch_gordan, config
 from e3nn_jax.util.sympy import sqrtQarray_to_sympy
 
-DEFAULT_SPHERICAL_HARMONICS_ALGORITHM = None
-
-
-def set_default_spherical_harmonics_algorithm(algorithm: Tuple[str]):
-    global DEFAULT_SPHERICAL_HARMONICS_ALGORITHM
-    DEFAULT_SPHERICAL_HARMONICS_ALGORITHM = algorithm
-
 
 def sh(
     irreps_out: Union[Irreps, int, Sequence[int]],
     input: jnp.ndarray,
     normalize: bool,
-    normalization: str = "integral",
+    normalization: str = None,
     *,
     algorithm: Tuple[str] = None,
 ) -> jnp.ndarray:
@@ -63,26 +56,26 @@ def sh(
 
     Args:
         irreps_out (`Irreps` or int or Sequence[int]): the output irreps
-        input (`jnp.ndarray`): cartesian coordinates
+        input (``jnp.ndarray``): cartesian coordinates
         normalize (bool): if True, the polynomials are restricted to the sphere
         normalization (str): normalization of the constant :math:`\text{cste}`. Default is 'integral'
         algorithm (Tuple[str]): algorithm to use for the computation. (legendre|recursive, dense|sparse, [custom_vjp])
 
     Returns:
-        `jnp.ndarray`: polynomials of the spherical harmonics
+        ``jnp.ndarray``: polynomials of the spherical harmonics
     """
-    input = IrrepsData.from_contiguous("1e", input)
-    return spherical_harmonics(irreps_out, input, normalize, normalization, algorithm=algorithm).contiguous
+    input = IrrepsArray("1e", input)
+    return spherical_harmonics(irreps_out, input, normalize, normalization, algorithm=algorithm).array
 
 
 def spherical_harmonics(
     irreps_out: Union[Irreps, int, Sequence[int]],
-    input: Union[IrrepsData, jnp.ndarray],
+    input: Union[IrrepsArray, jnp.ndarray],
     normalize: bool,
-    normalization: str = "integral",
+    normalization: str = None,
     *,
     algorithm: Tuple[str] = None,
-) -> IrrepsData:
+) -> IrrepsArray:
     r"""Spherical harmonics
 
     .. image:: https://user-images.githubusercontent.com/333780/79220728-dbe82c00-7e54-11ea-82c7-b3acbd9b2246.gif
@@ -101,9 +94,9 @@ def spherical_harmonics(
 
     .. math::
 
-        Y^{l+1}_i(x) &= \text{cste}(l) \; & C_{ijk} Y^l_j(x) x_k
+        Y^{l+1}_i(x) &= \text{cste}(l) \; C_{ijk} Y^l_j(x) x_k
 
-        \partial_k Y^{l+1}_i(x) &= \text{cste}(l) \; (l+1) & C_{ijk} Y^l_j(x)
+        \partial_k Y^{l+1}_i(x) &= \text{cste}(l) \; (l+1) C_{ijk} Y^l_j(x)
 
     Where :math:`C` are the `clebsch_gordan`.
 
@@ -117,24 +110,26 @@ def spherical_harmonics(
 
     Args:
         irreps_out (`Irreps` or int): output irreps
-        input (`IrrepsData` or `jnp.ndarray`): cartesian coordinates
+        input (`IrrepsArray` or ``jnp.ndarray``): cartesian coordinates
         normalize (bool): if True, the polynomials are restricted to the sphere
         normalization (str): normalization of the constant :math:`\text{cste}`. Default is 'integral'
         algorithm (Tuple[str]): algorithm to use for the computation. (legendre|recursive, dense|sparse, [custom_vjp])
 
     Returns:
-        `IrrepsData`: polynomials of the spherical harmonics
+        `IrrepsArray`: polynomials of the spherical harmonics
     """
+    if normalization is None:
+        normalization = config("spherical_harmonics_normalization")
     assert normalization in ["integral", "component", "norm"]
 
     if isinstance(irreps_out, int):
         l = irreps_out
-        assert isinstance(input, IrrepsData)
+        assert isinstance(input, IrrepsArray)
         [(mul, ir)] = input.irreps
         irreps_out = Irreps([(1, (l, ir.p**l))])
 
     if all(isinstance(l, int) for l in irreps_out):
-        assert isinstance(input, IrrepsData)
+        assert isinstance(input, IrrepsArray)
         [(mul, ir)] = input.irreps
         irreps_out = Irreps([(1, (l, ir.p**l)) for l in irreps_out])
 
@@ -144,22 +139,22 @@ def spherical_harmonics(
     assert len(set([p for _, (l, p) in irreps_out if l % 2 == 1])) <= 1
 
     if algorithm is None:
-        if DEFAULT_SPHERICAL_HARMONICS_ALGORITHM is None:
+        if config("spherical_harmonics_algorithm") == "automatic":
             if irreps_out.lmax <= 8:
                 algorithm = ("recursive", "sparse", "custom_vjp")
             else:
                 algorithm = ("legendre", "sparse", "custom_vjp")
         else:
-            algorithm = DEFAULT_SPHERICAL_HARMONICS_ALGORITHM
+            algorithm = config("spherical_harmonics_algorithm")
 
     assert all(keyword in ["legendre", "recursive", "dense", "sparse", "custom_vjp"] for keyword in algorithm)
 
-    if isinstance(input, IrrepsData):
+    if isinstance(input, IrrepsArray):
         [(mul, ir)] = input.irreps
         assert mul == 1
         assert ir.l == 1
         assert all([ir.p == p for _, (l, p) in irreps_out if l % 2 == 1])
-        x = input.contiguous
+        x = input.array
     else:
         x = input
 
@@ -170,7 +165,7 @@ def spherical_harmonics(
 
     sh = _jited_spherical_harmonics(tuple(ir.l for _, ir in irreps_out), x, normalization, algorithm)
     sh = [jnp.repeat(y[..., None, :], mul, -2) for (mul, ir), y in zip(irreps_out, sh)]
-    return IrrepsData.from_list(irreps_out, sh, x.shape[:-1])
+    return IrrepsArray.from_list(irreps_out, sh, x.shape[:-1])
 
 
 @partial(jax.jit, static_argnums=(0, 2, 3), inline=True)
@@ -195,6 +190,7 @@ def _spherical_harmonics(ls: Tuple[int, ...], x: jnp.ndarray, normalization: str
     raise ValueError("Unknown algorithm: must be 'legendre' or 'recursive'")
 
 
+@partial(jax.custom_jvp, nondiff_argnums=(0, 2, 3))
 @partial(jax.custom_vjp, nondiff_argnums=(0, 2, 3))
 def _custom_vjp_spherical_harmonics(
     ls: Tuple[int, ...], x: jnp.ndarray, normalization: str, algorithm: Tuple[str]
@@ -247,7 +243,42 @@ def _bwd(
     return (sum([h(l, r, g) if l > 0 else jnp.zeros_like(r, shape=r.shape[:-1] + (3,)) for l, r, g in zip(ls, res, grad)]),)
 
 
+def _jvp(
+    ls: Tuple[int, ...], normalization: str, algorithm: Tuple[str], primals: Tuple[jnp.ndarray], tangents: Tuple[jnp.ndarray]
+) -> List[jnp.ndarray]:
+    (x,) = primals
+    (x_dot,) = tangents
+
+    js = tuple(max(0, l - 1) for l in ls)
+    output = _custom_vjp_spherical_harmonics(ls + js, x, normalization, algorithm)
+    out, res = output[: len(ls)], output[len(ls) :]
+
+    def h(l: int, r: jnp.ndarray) -> jnp.ndarray:
+        w = clebsch_gordan(l - 1, l, 1)
+        if normalization == "norm":
+            w *= ((2 * l + 1) * l * (2 * l - 1)) ** 0.5
+        else:
+            w *= l**0.5 * (2 * l + 1)
+
+        if "dense" in algorithm:
+            return jnp.einsum("...i,...k,ijk->...j", r, x_dot, w)
+        if "sparse" in algorithm:
+            return jnp.stack(
+                [
+                    sum(
+                        [w[i, j, k] * r[..., i] * x_dot[..., k] for i in range(2 * l - 1) for k in range(3) if w[i, j, k] != 0]
+                    )
+                    for j in range(2 * l + 1)
+                ],
+                axis=-1,
+            )
+        raise ValueError("Unknown algorithm: must be 'dense' or 'sparse'")
+
+    return out, [h(l, r) if l > 0 else jnp.zeros_like(r) for l, r in zip(ls, res)]
+
+
 _custom_vjp_spherical_harmonics.defvjp(_fwd, _bwd)
+_custom_vjp_spherical_harmonics.defjvp(_jvp)
 
 
 def _recursive_spherical_harmonics(
