@@ -5,7 +5,7 @@ import jax
 import jax.numpy as jnp
 from e3nn_jax import (
     Irreps,
-    IrrepsData,
+    IrrepsArray,
     FunctionalTensorProduct,
     index_add,
     Linear,
@@ -39,10 +39,10 @@ class Convolution(hk.Module):
 
     @partial(jax.profiler.annotate_function, name="convolution")
     def __call__(
-        self, node_input: IrrepsData, edge_src, edge_dst, edge_attr: IrrepsData, node_attr=None, edge_scalar_attr=None
-    ) -> IrrepsData:
-        assert isinstance(node_input, IrrepsData)
-        assert isinstance(edge_attr, IrrepsData)
+        self, node_input: IrrepsArray, edge_src, edge_dst, edge_attr: IrrepsArray, node_attr=None, edge_scalar_attr=None
+    ) -> IrrepsArray:
+        assert isinstance(node_input, IrrepsArray)
+        assert isinstance(edge_attr, IrrepsArray)
 
         if node_attr is not None:
             tmp = jax.vmap(FullyConnectedTensorProduct(node_input.irreps + self.irreps_node_output))(node_input, node_attr)
@@ -51,7 +51,7 @@ class Convolution(hk.Module):
 
         node_features, node_self_out = tmp.split([len(node_input.irreps)])
 
-        edge_features = jax.tree_map(lambda x: x[edge_src], node_features)
+        edge_features = jax.tree_util.tree_map(lambda x: x[edge_src], node_features)
         del node_features
 
         ######################################################################################
@@ -94,7 +94,7 @@ class Convolution(hk.Module):
                             f"{tp.irreps_in1[ins.i_in1]},{tp.irreps_in2[ins.i_in2]},{tp.irreps_out[ins.i_out]}"
                         ),
                         shape=(weight.shape[1],) + ins.path_shape,
-                        init=hk.initializers.RandomNormal(),
+                        init=hk.initializers.RandomNormal(ins.weight_std),
                     )
                     / weight.shape[1] ** 0.5,
                     weight,
@@ -102,7 +102,7 @@ class Convolution(hk.Module):
                 for ins in tp.instructions
             ]
 
-            edge_features: IrrepsData = jax.vmap(tp.left_right, (0, 0, 0), 0)(weight, edge_features, edge_attr)
+            edge_features: IrrepsArray = jax.vmap(tp.left_right, (0, 0, 0), 0)(weight, edge_features, edge_attr)
         else:
             weight = [
                 hk.get_parameter(
@@ -111,16 +111,16 @@ class Convolution(hk.Module):
                         f"{tp.irreps_in1[ins.i_in1]},{tp.irreps_in2[ins.i_in2]},{tp.irreps_out[ins.i_out]}"
                     ),
                     shape=ins.path_shape,
-                    init=hk.initializers.RandomNormal(),
+                    init=hk.initializers.RandomNormal(ins.weight_std),
                 )
                 for ins in tp.instructions
             ]
-            edge_features: IrrepsData = jax.vmap(tp.left_right, (None, 0, 0), 0)(weight, edge_features, edge_attr)
+            edge_features: IrrepsArray = jax.vmap(tp.left_right, (None, 0, 0), 0)(weight, edge_features, edge_attr)
 
         edge_features = edge_features.remove_nones().simplify()
         ######################################################################################
 
-        node_features = jax.tree_map(lambda x: index_add(edge_dst, x, out_dim=node_input.shape[0]), edge_features)
+        node_features = jax.tree_util.tree_map(lambda x: index_add(edge_dst, x, out_dim=node_input.shape[0]), edge_features)
         node_features = node_features / self.num_neighbors**0.5
 
         ######################################################################################
@@ -136,4 +136,4 @@ class Convolution(hk.Module):
             c = jnp.cos(self.mixing_angle)
             s = jnp.sin(self.mixing_angle)
 
-        return c * node_self_out + s * node_conv_out
+        return node_self_out * c + node_conv_out * s
